#!/usr/local/bin/ruby -w
# ex:ts=8
# vim:sw=4:sts=4:tw=120
#
# $Id: supertime.rb,v 1.7 2003/03/27 08:44:53 eivind Exp $
#
class Time
    class <<self
	alias_method :supertime_local, :local
	#@@supertime_local = method(:local)
	#@@supertime_local = method(:local)
	def local(a, *rest)
	    raise ArgumentError, "Need an Integer as first parameter" unless a.kind_of? Integer
	    rest.unshift a
	    if rest.length > 7
		if rest[5] >= 1970 && rest[5] <= 2030
		    soffset = 0
		else
		    soffset = rest[5] - 2000
		    rest[5] = 2000
		end
	    else
		if rest[0] >= 1970 && rest[0] <= 2030
		    soffset = 0
		else
		    soffset = rest[0] - 2000
		    rest[0] = 2000
		end
	    end
	    time = supertime_local(*rest)
	    time.supertime_setoffset(soffset)
	    time
	end
	alias :mktime :local
	alias_method :supertime_gm, :gm
	def gm(a, *rest)
	    raise ArgumentError, "Need an Integer as first parameter" unless a.kind_of? Integer
	    rest.unshift a
	    if rest.length > 7
		if rest[5] >= 1970 && rest[5] <= 2030
		    soffset = 0
		else
		    soffset = rest[5] - 2000
		    rest[5] = 2000
		end
	    else
		if rest[0] >= 1970 && rest[0] <= 2030
		    soffset = 0
		else
		    soffset = rest[0] - 2000
		    rest[0] = 2000
		end
	    end
	    time = supertime_gm(*rest)
	    time.supertime_setoffset(soffset)
	    time
	end

	alias_method :supertime_at, :at
	def at(seconds, microseconds=0)
	    year = 0
	    if seconds >= 2147483648
		while seconds > 12622780800	# 400 years
		    year += 400
		    seconds -= 12622780800
		end
		# We are now at or before 2370.  We can chop 100s until we no longer can, knowing that all chopping of
		# 100s will cover centuries *without* leap years in the century year
		while seconds > 3360960000
		    year += 100
		    seconds -= 3360960000
		end
	    elsif seconds < 0
		# Bring us forward 400 years at a time if we are before 1601.  This means we have skipped the last
		# 400-year oddness, and can view centuries as NOT having a leap day.
		while seconds < -11644435584
		    seconds += 12622780800
		    year -= 400
		end
		# Up to -80 years, that is, at 1890
		while seconds < -2524521600
		    seconds += 3155673600
		    year -= 100
		end
		# Do a hickup around the last century
		if seconds < -2184683400
		    year -= 20
		    seconds += 631065600
		end
		# Skip 20 years at a time until we get inside range
		while seconds < 0
		    seconds += 631152000
		    year -= 20
		end
	    else
		year = 0
	    end
	    time = supertime_at(seconds)
	    time.supertime_setoffset(year)
	    time
	end
	alias_method :supertime_now, :now
	def now
	    tmp = supertime_now
	    tmp.supertime_setoffset(0)
	    tmp
	end
    end
    def supertime_setoffset(soffset)
	raise ArgumentError, "Can't set nil soffset" unless soffset
	@supertime_offset = soffset
    end
    alias_method :supertime_strftime, :strftime
    def strftime(format)
	supertime_strftime(format.gsub(/(^|[^%])(%%)*%c/, "\\1%a %b %d %H:%M:%S %Z %Y").gsub(/(^|[^%])(%%)*%Y/, sprintf("\\1%4d", year)))
    end
    alias_method :supertime_year, :year
    def year
	supertime_year + @supertime_offset
    end
    def +(number)
	Time.at(to_i + number)
    end
    def -(number)
	if number.kind_of? Numeric
	    return self + -number
	else
	    return to_i - number.to_i
	end
    end
    def to_s
	strftime("%c")
    end
    alias :inspect :to_s
end
